package ru.dbotthepony.kbox2d.api

import ru.dbotthepony.kvector.vector.Color
import ru.dbotthepony.kvector.vector.ndouble.Vector2d

/**
 * Implement and register this class with a b2World to provide debug drawing of physics
 * entities in your game.
 */
interface IDebugDraw {
	var drawShapes: Boolean
	var drawJoints: Boolean
	var drawAABB: Boolean
	var drawPairs: Boolean
	var drawCenterOfMess: Boolean

	/**
	 * Draw a closed polygon provided in CCW order.
	 */
	fun drawPolygon(vertices: List<Vector2d>, color: Color)

	/**
	 * Draw a solid closed polygon provided in CCW order.
	 */
	fun drawSolidPolygon(vertices: List<Vector2d>, color: Color)

	/**
	 * Draw a circle.
	 */
	fun drawCircle(center: Vector2d, radius: Double, color: Color)

	/**
	 * Draw a solid circle.
	 */
	fun drawSolidCircle(center: Vector2d, radius: Double, axis: Vector2d, color: Color)

	/**
	 * Draw a line segment.
	 */
	fun drawSegment(p1: Vector2d, p2: Vector2d, color: Color)

	/**
	 * Draw a transform. Choose your own length scale.
	 * @param xf a transform.
	 */
	fun drawTransform(xf: Transform)

	/**
	 * Draw a point.
	 */
	fun drawPoint(p: Vector2d, size: Double, color: Color)
}
