package ru.dbotthepony.kstarbound.defs.projectile

import ru.dbotthepony.kstarbound.defs.ConfigurableDefinition
import ru.dbotthepony.kstarbound.defs.ConfigurableTypeAdapter
import ru.dbotthepony.kstarbound.defs.IFrameGrid
import ru.dbotthepony.kstarbound.defs.ensureAbsolutePath
import ru.dbotthepony.kvector.vector.Color

class ConfigurableProjectile : ConfigurableDefinition<ConfigurableProjectile, ConfiguredProjectile>() {
	var projectileName: String? = null
	var physics: ProjectilePhysics = ProjectilePhysics.DEFAULT
	var damageKindImage: String? = null
	var damageType: String? = null
	var damageKind: String? = null

	var pointLight: Boolean = false
	var lightColor: Color? = null

	var onlyHitTerrain: Boolean = false
	var orientationLocked: Boolean = false

	var image: String? = null

	var timeToLive: Double = Double.POSITIVE_INFINITY
	var animationCycle: Double = Double.POSITIVE_INFINITY
	var bounces: Int = -1
	var frameNumber: Int = 1

	var scripts: Array<String> = Array(0) { "" }

	var hydrophobic: Boolean = false

	override fun configure(directory: String): ConfiguredProjectile {
		return ConfiguredProjectile(
			json = enroll(),
			projectileName = checkNotNull(projectileName) { "projectileName is null" },
			physics = physics,
			damageKindImage = damageKindImage,
			damageType = damageType,
			damageKind = damageKind,
			pointLight = pointLight,
			lightColor = lightColor,
			onlyHitTerrain = onlyHitTerrain,
			orientationLocked = orientationLocked,
			image = IFrameGrid.loadFrameStrip(ensureAbsolutePath(requireNotNull(image) { "image is null" }, directory), weak = true),
			timeToLive = timeToLive,
			animationCycle = animationCycle,
			bounces = bounces,
			frameNumber = frameNumber,
			scripts = scripts,
		)
	}

	companion object {
		val ADAPTER = ConfigurableTypeAdapter(
			::ConfigurableProjectile,
			ConfigurableProjectile::projectileName,
			ConfigurableProjectile::physics,
			ConfigurableProjectile::damageKindImage,
			ConfigurableProjectile::damageType,
			ConfigurableProjectile::damageKind,
			ConfigurableProjectile::pointLight,
			ConfigurableProjectile::lightColor,
			ConfigurableProjectile::onlyHitTerrain,
			ConfigurableProjectile::orientationLocked,
			ConfigurableProjectile::image,
			ConfigurableProjectile::timeToLive,
			ConfigurableProjectile::animationCycle,
			ConfigurableProjectile::bounces,
			ConfigurableProjectile::frameNumber,
			ConfigurableProjectile::scripts,
		)
	}
}
