package ru.dbotthepony.kstarbound.world.entities

import ru.dbotthepony.kbox2d.api.FixtureDef
import ru.dbotthepony.kbox2d.collision.shapes.PolygonShape
import ru.dbotthepony.kbox2d.dynamics.B2Fixture
import ru.dbotthepony.kstarbound.world.World
import ru.dbotthepony.kvector.util2d.AABB
import ru.dbotthepony.kvector.vector.ndouble.Vector2d

class PlayerMovementController(entity: PlayerEntity) : WalkableMovementController<PlayerEntity>(entity) {
	public override var moveDirection = Move.STAND_STILL
	private var bodyFixture: B2Fixture

	override var isDucked: Boolean = false
		set(value) {
			if (value == field)
				return

			field = value

			bodyFixture.destroy()

			if (value) {
				bodyFixture = body.createFixture(FixtureDef(
					shape = DUCKING,
					friction = 0.4,
					density = 1.9,
				))
			} else {
				bodyFixture = body.createFixture(FixtureDef(
					shape = STANDING,
					friction = 0.4,
					density = 1.9,
				))
			}

			body.isAwake = true
		}

	override fun canUnDuck(): Boolean {
		return world.isSpaceEmptyFromTiles(STANDING_AABB + position)
	}

	init {
		bodyFixture = body.createFixture(FixtureDef(
			shape = STANDING,
			friction = 0.4,
			density = 1.9,
		))
	}

	companion object {
		private val STANDING = PolygonShape().also { it.setAsBox(0.9, 1.8) }
		private val STANDING_AABB = STANDING.computeAABB(0)
		private val DUCKING = PolygonShape().also { it.setAsBox(0.9, 0.9, Vector2d(y = -0.9), 0.0) }
		private val DUCKING_AABB = DUCKING.computeAABB(0)
	}
}

/**
 * Физический аватар игрока в мире
 */
open class PlayerEntity(world: World<*, *>) : AliveWalkingEntity(world) {
	override val movement = PlayerMovementController(this)

	override fun thinkAI(delta: Double) {

	}
}
